from wand.image import Image


def is_valid(adjacents, cluster):
    valid = True
    for p0 in cluster:
        connection = []
        for p1 in cluster:
            if p0 == p1:
                continue
            connected = p0 in adjacents[p1]
            connection.append(connected)
        if not True in connection:
            valid = False
            print(f"Not valid fragment, piece {p0} is not connected")
            break
    print("The fragment is valid!")
    return valid


def index2d(list2d, value):
    return next(
        (i, j) for i, lst in enumerate(list2d) for j, x in enumerate(lst) if x == value
    )


def create(pieces, cluster, name, side):
    schema = []
    for ID in cluster:
        piece = {"ID": ID, "coords": index2d(pieces, ID)}
        schema.append(piece)

    left = min(schema, key=lambda piece: piece["coords"][0])["coords"][0]
    right = max(schema, key=lambda piece: piece["coords"][0])["coords"][0]
    top = min(schema, key=lambda piece: piece["coords"][1])["coords"][1]
    bottom = max(schema, key=lambda piece: piece["coords"][1])["coords"][1]

    width = (right - left + 1) * side
    height = (bottom - top + 1) * side
    print(f"Final cluster will be {width} x {height} px")

    with Image(width=width, height=height) as img:
        cluster_name = "cluster"
        for piece in schema:
            cluster_name = f"{cluster_name}-{piece['ID']}"
            coords = piece["coords"]
            with Image(
                width=side,
                height=side,
                filename=f"chaospuzzles/puzzles/{name}/pieces/{name}-{coords[0]}-{coords[1]}.jpg",
            ) as tile:
                l = (piece["coords"][0] - left) * side
                t = (piece["coords"][1] - top) * side
                img.composite(tile, left=l, top=t)
                print(f"Composed with tile {piece['ID']}, at ({l}, {t})")
        # img.save(filename=f"chaospuzzles/puzzles/{name}/clusters/{cluster_name}.png")
        img.save(filename=f"chaospuzzles/static/img/{name}-{cluster_name}.png")
        print("Done!")
    return {
        "name": f"{cluster_name}.png",
        "origin": (left, top),
        "size": (right - left + 1, bottom - top + 1),
    }
